# DO NOT EDIT! This file is automatically generated
import marshmallow
import marshmallow_enum

from commercetools import helpers, types
from commercetools._schemas._common import (
    BaseResourceSchema,
    LocalizedStringField,
    ReferenceSchema,
    ResourceIdentifierSchema,
)
from commercetools._schemas._order import OrderSchema, StagedOrderUpdateActionSchema
from commercetools._schemas._type import FieldContainerField

__all__ = [
    "OrderEditAddStagedActionActionSchema",
    "OrderEditAppliedSchema",
    "OrderEditApplySchema",
    "OrderEditDraftSchema",
    "OrderEditNotProcessedSchema",
    "OrderEditPagedQueryResponseSchema",
    "OrderEditPreviewFailureSchema",
    "OrderEditPreviewSuccessSchema",
    "OrderEditReferenceSchema",
    "OrderEditResourceIdentifierSchema",
    "OrderEditResultSchema",
    "OrderEditSchema",
    "OrderEditSetCommentActionSchema",
    "OrderEditSetCustomFieldActionSchema",
    "OrderEditSetCustomTypeActionSchema",
    "OrderEditSetKeyActionSchema",
    "OrderEditSetStagedActionsActionSchema",
    "OrderEditUpdateActionSchema",
    "OrderEditUpdateSchema",
    "OrderExcerptSchema",
    "StagedOrderAddCustomLineItemActionSchema",
    "StagedOrderAddDeliveryActionSchema",
    "StagedOrderAddDiscountCodeActionSchema",
    "StagedOrderAddItemShippingAddressActionSchema",
    "StagedOrderAddLineItemActionSchema",
    "StagedOrderAddParcelToDeliveryActionSchema",
    "StagedOrderAddPaymentActionSchema",
    "StagedOrderAddReturnInfoActionSchema",
    "StagedOrderAddShoppingListActionSchema",
    "StagedOrderChangeCustomLineItemMoneyActionSchema",
    "StagedOrderChangeCustomLineItemQuantityActionSchema",
    "StagedOrderChangeLineItemQuantityActionSchema",
    "StagedOrderChangeOrderStateActionSchema",
    "StagedOrderChangePaymentStateActionSchema",
    "StagedOrderChangeShipmentStateActionSchema",
    "StagedOrderChangeTaxCalculationModeActionSchema",
    "StagedOrderChangeTaxModeActionSchema",
    "StagedOrderChangeTaxRoundingModeActionSchema",
    "StagedOrderImportCustomLineItemStateActionSchema",
    "StagedOrderImportLineItemStateActionSchema",
    "StagedOrderRemoveCustomLineItemActionSchema",
    "StagedOrderRemoveDeliveryActionSchema",
    "StagedOrderRemoveDiscountCodeActionSchema",
    "StagedOrderRemoveItemShippingAddressActionSchema",
    "StagedOrderRemoveLineItemActionSchema",
    "StagedOrderRemoveParcelFromDeliveryActionSchema",
    "StagedOrderRemovePaymentActionSchema",
    "StagedOrderSchema",
    "StagedOrderSetBillingAddressActionSchema",
    "StagedOrderSetCountryActionSchema",
    "StagedOrderSetCustomFieldActionSchema",
    "StagedOrderSetCustomLineItemCustomFieldActionSchema",
    "StagedOrderSetCustomLineItemCustomTypeActionSchema",
    "StagedOrderSetCustomLineItemShippingDetailsActionSchema",
    "StagedOrderSetCustomLineItemTaxAmountActionSchema",
    "StagedOrderSetCustomLineItemTaxRateActionSchema",
    "StagedOrderSetCustomShippingMethodActionSchema",
    "StagedOrderSetCustomTypeActionSchema",
    "StagedOrderSetCustomerEmailActionSchema",
    "StagedOrderSetCustomerGroupActionSchema",
    "StagedOrderSetCustomerIdActionSchema",
    "StagedOrderSetDeliveryAddressActionSchema",
    "StagedOrderSetDeliveryItemsActionSchema",
    "StagedOrderSetLineItemCustomFieldActionSchema",
    "StagedOrderSetLineItemCustomTypeActionSchema",
    "StagedOrderSetLineItemDistributionChannelActionSchema",
    "StagedOrderSetLineItemPriceActionSchema",
    "StagedOrderSetLineItemShippingDetailsActionSchema",
    "StagedOrderSetLineItemTaxAmountActionSchema",
    "StagedOrderSetLineItemTaxRateActionSchema",
    "StagedOrderSetLineItemTotalPriceActionSchema",
    "StagedOrderSetLocaleActionSchema",
    "StagedOrderSetOrderNumberActionSchema",
    "StagedOrderSetOrderTotalTaxActionSchema",
    "StagedOrderSetParcelItemsActionSchema",
    "StagedOrderSetParcelMeasurementsActionSchema",
    "StagedOrderSetParcelTrackingDataActionSchema",
    "StagedOrderSetReturnPaymentStateActionSchema",
    "StagedOrderSetReturnShipmentStateActionSchema",
    "StagedOrderSetShippingAddressActionSchema",
    "StagedOrderSetShippingAddressAndCustomShippingMethodActionSchema",
    "StagedOrderSetShippingAddressAndShippingMethodActionSchema",
    "StagedOrderSetShippingMethodActionSchema",
    "StagedOrderSetShippingMethodTaxAmountActionSchema",
    "StagedOrderSetShippingMethodTaxRateActionSchema",
    "StagedOrderSetShippingRateInputActionSchema",
    "StagedOrderTransitionCustomLineItemStateActionSchema",
    "StagedOrderTransitionLineItemStateActionSchema",
    "StagedOrderTransitionStateActionSchema",
    "StagedOrderUpdateItemShippingAddressActionSchema",
    "StagedOrderUpdateSyncInfoActionSchema",
]


class OrderEditApplySchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditApply`."""

    edit_version = marshmallow.fields.Integer(allow_none=True, data_key="editVersion")
    resource_version = marshmallow.fields.Integer(
        allow_none=True, data_key="resourceVersion"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.OrderEditApply(**data)


class OrderEditDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditDraft`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)
    resource = helpers.LazyNestedField(
        nested="commercetools._schemas._order.OrderReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    staged_actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addCustomLineItem": "commercetools._schemas._order_edit.StagedOrderAddCustomLineItemActionSchema",
                "addDelivery": "commercetools._schemas._order_edit.StagedOrderAddDeliveryActionSchema",
                "addDiscountCode": "commercetools._schemas._order_edit.StagedOrderAddDiscountCodeActionSchema",
                "addItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderAddItemShippingAddressActionSchema",
                "addLineItem": "commercetools._schemas._order_edit.StagedOrderAddLineItemActionSchema",
                "addParcelToDelivery": "commercetools._schemas._order_edit.StagedOrderAddParcelToDeliveryActionSchema",
                "addPayment": "commercetools._schemas._order_edit.StagedOrderAddPaymentActionSchema",
                "addReturnInfo": "commercetools._schemas._order_edit.StagedOrderAddReturnInfoActionSchema",
                "addShoppingList": "commercetools._schemas._order_edit.StagedOrderAddShoppingListActionSchema",
                "changeCustomLineItemMoney": "commercetools._schemas._order_edit.StagedOrderChangeCustomLineItemMoneyActionSchema",
                "changeCustomLineItemQuantity": "commercetools._schemas._order_edit.StagedOrderChangeCustomLineItemQuantityActionSchema",
                "changeLineItemQuantity": "commercetools._schemas._order_edit.StagedOrderChangeLineItemQuantityActionSchema",
                "changeOrderState": "commercetools._schemas._order_edit.StagedOrderChangeOrderStateActionSchema",
                "changePaymentState": "commercetools._schemas._order_edit.StagedOrderChangePaymentStateActionSchema",
                "changeShipmentState": "commercetools._schemas._order_edit.StagedOrderChangeShipmentStateActionSchema",
                "changeTaxCalculationMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxCalculationModeActionSchema",
                "changeTaxMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxModeActionSchema",
                "changeTaxRoundingMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxRoundingModeActionSchema",
                "importCustomLineItemState": "commercetools._schemas._order_edit.StagedOrderImportCustomLineItemStateActionSchema",
                "importLineItemState": "commercetools._schemas._order_edit.StagedOrderImportLineItemStateActionSchema",
                "removeCustomLineItem": "commercetools._schemas._order_edit.StagedOrderRemoveCustomLineItemActionSchema",
                "removeDelivery": "commercetools._schemas._order_edit.StagedOrderRemoveDeliveryActionSchema",
                "removeDiscountCode": "commercetools._schemas._order_edit.StagedOrderRemoveDiscountCodeActionSchema",
                "removeItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderRemoveItemShippingAddressActionSchema",
                "removeLineItem": "commercetools._schemas._order_edit.StagedOrderRemoveLineItemActionSchema",
                "removeParcelFromDelivery": "commercetools._schemas._order_edit.StagedOrderRemoveParcelFromDeliveryActionSchema",
                "removePayment": "commercetools._schemas._order_edit.StagedOrderRemovePaymentActionSchema",
                "setBillingAddress": "commercetools._schemas._order_edit.StagedOrderSetBillingAddressActionSchema",
                "setCountry": "commercetools._schemas._order_edit.StagedOrderSetCountryActionSchema",
                "setCustomField": "commercetools._schemas._order_edit.StagedOrderSetCustomFieldActionSchema",
                "setCustomLineItemCustomField": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemCustomFieldActionSchema",
                "setCustomLineItemCustomType": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemCustomTypeActionSchema",
                "setCustomLineItemShippingDetails": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemShippingDetailsActionSchema",
                "setCustomLineItemTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemTaxAmountActionSchema",
                "setCustomLineItemTaxRate": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemTaxRateActionSchema",
                "setCustomShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetCustomShippingMethodActionSchema",
                "setCustomType": "commercetools._schemas._order_edit.StagedOrderSetCustomTypeActionSchema",
                "setCustomerEmail": "commercetools._schemas._order_edit.StagedOrderSetCustomerEmailActionSchema",
                "setCustomerGroup": "commercetools._schemas._order_edit.StagedOrderSetCustomerGroupActionSchema",
                "setCustomerId": "commercetools._schemas._order_edit.StagedOrderSetCustomerIdActionSchema",
                "setDeliveryAddress": "commercetools._schemas._order_edit.StagedOrderSetDeliveryAddressActionSchema",
                "setDeliveryItems": "commercetools._schemas._order_edit.StagedOrderSetDeliveryItemsActionSchema",
                "setLineItemCustomField": "commercetools._schemas._order_edit.StagedOrderSetLineItemCustomFieldActionSchema",
                "setLineItemCustomType": "commercetools._schemas._order_edit.StagedOrderSetLineItemCustomTypeActionSchema",
                "setLineItemDistributionChannel": "commercetools._schemas._order_edit.StagedOrderSetLineItemDistributionChannelActionSchema",
                "setLineItemPrice": "commercetools._schemas._order_edit.StagedOrderSetLineItemPriceActionSchema",
                "setLineItemShippingDetails": "commercetools._schemas._order_edit.StagedOrderSetLineItemShippingDetailsActionSchema",
                "setLineItemTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetLineItemTaxAmountActionSchema",
                "setLineItemTaxRate": "commercetools._schemas._order_edit.StagedOrderSetLineItemTaxRateActionSchema",
                "setLineItemTotalPrice": "commercetools._schemas._order_edit.StagedOrderSetLineItemTotalPriceActionSchema",
                "setLocale": "commercetools._schemas._order_edit.StagedOrderSetLocaleActionSchema",
                "setOrderNumber": "commercetools._schemas._order_edit.StagedOrderSetOrderNumberActionSchema",
                "setOrderTotalTax": "commercetools._schemas._order_edit.StagedOrderSetOrderTotalTaxActionSchema",
                "setParcelItems": "commercetools._schemas._order_edit.StagedOrderSetParcelItemsActionSchema",
                "setParcelMeasurements": "commercetools._schemas._order_edit.StagedOrderSetParcelMeasurementsActionSchema",
                "setParcelTrackingData": "commercetools._schemas._order_edit.StagedOrderSetParcelTrackingDataActionSchema",
                "setReturnPaymentState": "commercetools._schemas._order_edit.StagedOrderSetReturnPaymentStateActionSchema",
                "setReturnShipmentState": "commercetools._schemas._order_edit.StagedOrderSetReturnShipmentStateActionSchema",
                "setShippingAddress": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressActionSchema",
                "setShippingAddressAndCustomShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressAndCustomShippingMethodActionSchema",
                "setShippingAddressAndShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressAndShippingMethodActionSchema",
                "setShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodActionSchema",
                "setShippingMethodTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodTaxAmountActionSchema",
                "setShippingMethodTaxRate": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodTaxRateActionSchema",
                "setShippingRateInput": "commercetools._schemas._order_edit.StagedOrderSetShippingRateInputActionSchema",
                "transitionCustomLineItemState": "commercetools._schemas._order_edit.StagedOrderTransitionCustomLineItemStateActionSchema",
                "transitionLineItemState": "commercetools._schemas._order_edit.StagedOrderTransitionLineItemStateActionSchema",
                "transitionState": "commercetools._schemas._order_edit.StagedOrderTransitionStateActionSchema",
                "updateItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderUpdateItemShippingAddressActionSchema",
                "updateSyncInfo": "commercetools._schemas._order_edit.StagedOrderUpdateSyncInfoActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        missing=None,
        data_key="stagedActions",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    comment = marshmallow.fields.String(allow_none=True, missing=None)
    dry_run = marshmallow.fields.Bool(allow_none=True, missing=None, data_key="dryRun")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.OrderEditDraft(**data)


class OrderEditPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._order_edit.OrderEditSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.OrderEditPagedQueryResponse(**data)


class OrderEditReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._order_edit.OrderEditSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.OrderEditReference(**data)


class OrderEditResourceIdentifierSchema(ResourceIdentifierSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditResourceIdentifier`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.OrderEditResourceIdentifier(**data)


class OrderEditResultSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditResult`."""

    type = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.OrderEditResult(**data)


class OrderEditSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEdit`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    key = marshmallow.fields.String(allow_none=True, missing=None)
    resource = helpers.LazyNestedField(
        nested="commercetools._schemas._order.OrderReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    staged_actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addCustomLineItem": "commercetools._schemas._order_edit.StagedOrderAddCustomLineItemActionSchema",
                "addDelivery": "commercetools._schemas._order_edit.StagedOrderAddDeliveryActionSchema",
                "addDiscountCode": "commercetools._schemas._order_edit.StagedOrderAddDiscountCodeActionSchema",
                "addItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderAddItemShippingAddressActionSchema",
                "addLineItem": "commercetools._schemas._order_edit.StagedOrderAddLineItemActionSchema",
                "addParcelToDelivery": "commercetools._schemas._order_edit.StagedOrderAddParcelToDeliveryActionSchema",
                "addPayment": "commercetools._schemas._order_edit.StagedOrderAddPaymentActionSchema",
                "addReturnInfo": "commercetools._schemas._order_edit.StagedOrderAddReturnInfoActionSchema",
                "addShoppingList": "commercetools._schemas._order_edit.StagedOrderAddShoppingListActionSchema",
                "changeCustomLineItemMoney": "commercetools._schemas._order_edit.StagedOrderChangeCustomLineItemMoneyActionSchema",
                "changeCustomLineItemQuantity": "commercetools._schemas._order_edit.StagedOrderChangeCustomLineItemQuantityActionSchema",
                "changeLineItemQuantity": "commercetools._schemas._order_edit.StagedOrderChangeLineItemQuantityActionSchema",
                "changeOrderState": "commercetools._schemas._order_edit.StagedOrderChangeOrderStateActionSchema",
                "changePaymentState": "commercetools._schemas._order_edit.StagedOrderChangePaymentStateActionSchema",
                "changeShipmentState": "commercetools._schemas._order_edit.StagedOrderChangeShipmentStateActionSchema",
                "changeTaxCalculationMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxCalculationModeActionSchema",
                "changeTaxMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxModeActionSchema",
                "changeTaxRoundingMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxRoundingModeActionSchema",
                "importCustomLineItemState": "commercetools._schemas._order_edit.StagedOrderImportCustomLineItemStateActionSchema",
                "importLineItemState": "commercetools._schemas._order_edit.StagedOrderImportLineItemStateActionSchema",
                "removeCustomLineItem": "commercetools._schemas._order_edit.StagedOrderRemoveCustomLineItemActionSchema",
                "removeDelivery": "commercetools._schemas._order_edit.StagedOrderRemoveDeliveryActionSchema",
                "removeDiscountCode": "commercetools._schemas._order_edit.StagedOrderRemoveDiscountCodeActionSchema",
                "removeItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderRemoveItemShippingAddressActionSchema",
                "removeLineItem": "commercetools._schemas._order_edit.StagedOrderRemoveLineItemActionSchema",
                "removeParcelFromDelivery": "commercetools._schemas._order_edit.StagedOrderRemoveParcelFromDeliveryActionSchema",
                "removePayment": "commercetools._schemas._order_edit.StagedOrderRemovePaymentActionSchema",
                "setBillingAddress": "commercetools._schemas._order_edit.StagedOrderSetBillingAddressActionSchema",
                "setCountry": "commercetools._schemas._order_edit.StagedOrderSetCountryActionSchema",
                "setCustomField": "commercetools._schemas._order_edit.StagedOrderSetCustomFieldActionSchema",
                "setCustomLineItemCustomField": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemCustomFieldActionSchema",
                "setCustomLineItemCustomType": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemCustomTypeActionSchema",
                "setCustomLineItemShippingDetails": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemShippingDetailsActionSchema",
                "setCustomLineItemTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemTaxAmountActionSchema",
                "setCustomLineItemTaxRate": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemTaxRateActionSchema",
                "setCustomShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetCustomShippingMethodActionSchema",
                "setCustomType": "commercetools._schemas._order_edit.StagedOrderSetCustomTypeActionSchema",
                "setCustomerEmail": "commercetools._schemas._order_edit.StagedOrderSetCustomerEmailActionSchema",
                "setCustomerGroup": "commercetools._schemas._order_edit.StagedOrderSetCustomerGroupActionSchema",
                "setCustomerId": "commercetools._schemas._order_edit.StagedOrderSetCustomerIdActionSchema",
                "setDeliveryAddress": "commercetools._schemas._order_edit.StagedOrderSetDeliveryAddressActionSchema",
                "setDeliveryItems": "commercetools._schemas._order_edit.StagedOrderSetDeliveryItemsActionSchema",
                "setLineItemCustomField": "commercetools._schemas._order_edit.StagedOrderSetLineItemCustomFieldActionSchema",
                "setLineItemCustomType": "commercetools._schemas._order_edit.StagedOrderSetLineItemCustomTypeActionSchema",
                "setLineItemDistributionChannel": "commercetools._schemas._order_edit.StagedOrderSetLineItemDistributionChannelActionSchema",
                "setLineItemPrice": "commercetools._schemas._order_edit.StagedOrderSetLineItemPriceActionSchema",
                "setLineItemShippingDetails": "commercetools._schemas._order_edit.StagedOrderSetLineItemShippingDetailsActionSchema",
                "setLineItemTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetLineItemTaxAmountActionSchema",
                "setLineItemTaxRate": "commercetools._schemas._order_edit.StagedOrderSetLineItemTaxRateActionSchema",
                "setLineItemTotalPrice": "commercetools._schemas._order_edit.StagedOrderSetLineItemTotalPriceActionSchema",
                "setLocale": "commercetools._schemas._order_edit.StagedOrderSetLocaleActionSchema",
                "setOrderNumber": "commercetools._schemas._order_edit.StagedOrderSetOrderNumberActionSchema",
                "setOrderTotalTax": "commercetools._schemas._order_edit.StagedOrderSetOrderTotalTaxActionSchema",
                "setParcelItems": "commercetools._schemas._order_edit.StagedOrderSetParcelItemsActionSchema",
                "setParcelMeasurements": "commercetools._schemas._order_edit.StagedOrderSetParcelMeasurementsActionSchema",
                "setParcelTrackingData": "commercetools._schemas._order_edit.StagedOrderSetParcelTrackingDataActionSchema",
                "setReturnPaymentState": "commercetools._schemas._order_edit.StagedOrderSetReturnPaymentStateActionSchema",
                "setReturnShipmentState": "commercetools._schemas._order_edit.StagedOrderSetReturnShipmentStateActionSchema",
                "setShippingAddress": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressActionSchema",
                "setShippingAddressAndCustomShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressAndCustomShippingMethodActionSchema",
                "setShippingAddressAndShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressAndShippingMethodActionSchema",
                "setShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodActionSchema",
                "setShippingMethodTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodTaxAmountActionSchema",
                "setShippingMethodTaxRate": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodTaxRateActionSchema",
                "setShippingRateInput": "commercetools._schemas._order_edit.StagedOrderSetShippingRateInputActionSchema",
                "transitionCustomLineItemState": "commercetools._schemas._order_edit.StagedOrderTransitionCustomLineItemStateActionSchema",
                "transitionLineItemState": "commercetools._schemas._order_edit.StagedOrderTransitionLineItemStateActionSchema",
                "transitionState": "commercetools._schemas._order_edit.StagedOrderTransitionStateActionSchema",
                "updateItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderUpdateItemShippingAddressActionSchema",
                "updateSyncInfo": "commercetools._schemas._order_edit.StagedOrderUpdateSyncInfoActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        data_key="stagedActions",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    result = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "Applied": "commercetools._schemas._order_edit.OrderEditAppliedSchema",
            "NotProcessed": "commercetools._schemas._order_edit.OrderEditNotProcessedSchema",
            "PreviewFailure": "commercetools._schemas._order_edit.OrderEditPreviewFailureSchema",
            "PreviewSuccess": "commercetools._schemas._order_edit.OrderEditPreviewSuccessSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    comment = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.OrderEdit(**data)


class OrderEditUpdateActionSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditUpdateAction`."""

    action = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.OrderEditUpdateAction(**data)


class OrderEditUpdateSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditUpdate`."""

    version = marshmallow.fields.Integer(allow_none=True)
    actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addStagedAction": "commercetools._schemas._order_edit.OrderEditAddStagedActionActionSchema",
                "setComment": "commercetools._schemas._order_edit.OrderEditSetCommentActionSchema",
                "setCustomField": "commercetools._schemas._order_edit.OrderEditSetCustomFieldActionSchema",
                "setCustomType": "commercetools._schemas._order_edit.OrderEditSetCustomTypeActionSchema",
                "setKey": "commercetools._schemas._order_edit.OrderEditSetKeyActionSchema",
                "setStagedActions": "commercetools._schemas._order_edit.OrderEditSetStagedActionsActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        )
    )
    dry_run = marshmallow.fields.Bool(allow_none=True, missing=None, data_key="dryRun")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.OrderEditUpdate(**data)


class OrderExcerptSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.OrderExcerpt`."""

    total_price = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "centPrecision": "commercetools._schemas._common.CentPrecisionMoneySchema",
            "highPrecision": "commercetools._schemas._common.HighPrecisionMoneySchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="totalPrice",
    )
    taxed_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxedPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxedPrice",
    )
    version = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.OrderExcerpt(**data)


class StagedOrderAddCustomLineItemActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddCustomLineItemAction`."""

    money = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    name = LocalizedStringField(allow_none=True)
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    slug = marshmallow.fields.String(allow_none=True)
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxCategory",
    )
    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddCustomLineItemAction(**data)


class StagedOrderAddDeliveryActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddDeliveryAction`."""

    items = helpers.LazyNestedField(
        nested="commercetools._schemas._order.DeliveryItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )
    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    parcels = helpers.LazyNestedField(
        nested="commercetools._schemas._order.ParcelDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddDeliveryAction(**data)


class StagedOrderAddDiscountCodeActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddDiscountCodeAction`."""

    code = marshmallow.fields.String(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddDiscountCodeAction(**data)


class StagedOrderAddItemShippingAddressActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddItemShippingAddressAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddItemShippingAddressAction(**data)


class StagedOrderAddLineItemActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddLineItemAction`."""

    custom = helpers.LazyNestedField(
        nested="commercetools._schemas._type.CustomFieldsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )
    product_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="productId"
    )
    variant_id = marshmallow.fields.Integer(
        allow_none=True, missing=None, data_key="variantId"
    )
    sku = marshmallow.fields.String(allow_none=True, missing=None)
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    added_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="addedAt"
    )
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddLineItemAction(**data)


class StagedOrderAddParcelToDeliveryActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddParcelToDeliveryAction`."""

    delivery_id = marshmallow.fields.String(allow_none=True, data_key="deliveryId")
    measurements = helpers.LazyNestedField(
        nested="commercetools._schemas._order.ParcelMeasurementsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    tracking_data = helpers.LazyNestedField(
        nested="commercetools._schemas._order.TrackingDataSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="trackingData",
    )
    items = helpers.LazyNestedField(
        nested="commercetools._schemas._order.DeliveryItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddParcelToDeliveryAction(**data)


class StagedOrderAddPaymentActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddPaymentAction`."""

    payment = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddPaymentAction(**data)


class StagedOrderAddReturnInfoActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddReturnInfoAction`."""

    return_tracking_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="returnTrackingId"
    )
    items = helpers.LazyNestedField(
        nested="commercetools._schemas._order.ReturnItemDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )
    return_date = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="returnDate"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddReturnInfoAction(**data)


class StagedOrderAddShoppingListActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderAddShoppingListAction`."""

    shopping_list = helpers.LazyNestedField(
        nested="commercetools._schemas._shopping_list.ShoppingListResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="shoppingList",
    )
    supply_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="supplyChannel",
    )
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderAddShoppingListAction(**data)


class StagedOrderChangeCustomLineItemMoneyActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangeCustomLineItemMoneyAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    money = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangeCustomLineItemMoneyAction(**data)


class StagedOrderChangeCustomLineItemQuantityActionSchema(
    StagedOrderUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangeCustomLineItemQuantityAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    quantity = marshmallow.fields.Integer(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangeCustomLineItemQuantityAction(**data)


class StagedOrderChangeLineItemQuantityActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangeLineItemQuantityAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    quantity = marshmallow.fields.Integer(allow_none=True)
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangeLineItemQuantityAction(**data)


class StagedOrderChangeOrderStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangeOrderStateAction`."""

    order_state = marshmallow_enum.EnumField(
        types.OrderState, by_value=True, data_key="orderState"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangeOrderStateAction(**data)


class StagedOrderChangePaymentStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangePaymentStateAction`."""

    payment_state = marshmallow_enum.EnumField(
        types.PaymentState, by_value=True, missing=None, data_key="paymentState"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangePaymentStateAction(**data)


class StagedOrderChangeShipmentStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangeShipmentStateAction`."""

    shipment_state = marshmallow_enum.EnumField(
        types.ShipmentState, by_value=True, missing=None, data_key="shipmentState"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangeShipmentStateAction(**data)


class StagedOrderChangeTaxCalculationModeActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangeTaxCalculationModeAction`."""

    tax_calculation_mode = marshmallow_enum.EnumField(
        types.TaxCalculationMode, by_value=True, data_key="taxCalculationMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangeTaxCalculationModeAction(**data)


class StagedOrderChangeTaxModeActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangeTaxModeAction`."""

    tax_mode = marshmallow_enum.EnumField(
        types.TaxMode, by_value=True, data_key="taxMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangeTaxModeAction(**data)


class StagedOrderChangeTaxRoundingModeActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderChangeTaxRoundingModeAction`."""

    tax_rounding_mode = marshmallow_enum.EnumField(
        types.RoundingMode, by_value=True, data_key="taxRoundingMode"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderChangeTaxRoundingModeAction(**data)


class StagedOrderImportCustomLineItemStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderImportCustomLineItemStateAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    state = helpers.LazyNestedField(
        nested="commercetools._schemas._order.ItemStateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderImportCustomLineItemStateAction(**data)


class StagedOrderImportLineItemStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderImportLineItemStateAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    state = helpers.LazyNestedField(
        nested="commercetools._schemas._order.ItemStateSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderImportLineItemStateAction(**data)


class StagedOrderRemoveCustomLineItemActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderRemoveCustomLineItemAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderRemoveCustomLineItemAction(**data)


class StagedOrderRemoveDeliveryActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderRemoveDeliveryAction`."""

    delivery_id = marshmallow.fields.String(allow_none=True, data_key="deliveryId")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderRemoveDeliveryAction(**data)


class StagedOrderRemoveDiscountCodeActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderRemoveDiscountCodeAction`."""

    discount_code = helpers.LazyNestedField(
        nested="commercetools._schemas._discount_code.DiscountCodeReferenceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="discountCode",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderRemoveDiscountCodeAction(**data)


class StagedOrderRemoveItemShippingAddressActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderRemoveItemShippingAddressAction`."""

    address_key = marshmallow.fields.String(allow_none=True, data_key="addressKey")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderRemoveItemShippingAddressAction(**data)


class StagedOrderRemoveLineItemActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderRemoveLineItemAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    quantity = marshmallow.fields.Integer(allow_none=True, missing=None)
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )
    shipping_details_to_remove = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetailsToRemove",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderRemoveLineItemAction(**data)


class StagedOrderRemoveParcelFromDeliveryActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderRemoveParcelFromDeliveryAction`."""

    parcel_id = marshmallow.fields.String(allow_none=True, data_key="parcelId")

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderRemoveParcelFromDeliveryAction(**data)


class StagedOrderRemovePaymentActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderRemovePaymentAction`."""

    payment = helpers.LazyNestedField(
        nested="commercetools._schemas._payment.PaymentResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderRemovePaymentAction(**data)


class StagedOrderSchema(OrderSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrder`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.StagedOrder(**data)


class StagedOrderSetBillingAddressActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetBillingAddressAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetBillingAddressAction(**data)


class StagedOrderSetCountryActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCountryAction`."""

    country = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCountryAction(**data)


class StagedOrderSetCustomFieldActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomFieldAction(**data)


class StagedOrderSetCustomLineItemCustomFieldActionSchema(
    StagedOrderUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomLineItemCustomFieldAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomLineItemCustomFieldAction(**data)


class StagedOrderSetCustomLineItemCustomTypeActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomLineItemCustomTypeAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomLineItemCustomTypeAction(**data)


class StagedOrderSetCustomLineItemShippingDetailsActionSchema(
    StagedOrderUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomLineItemShippingDetailsAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomLineItemShippingDetailsAction(**data)


class StagedOrderSetCustomLineItemTaxAmountActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomLineItemTaxAmountAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    external_tax_amount = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxAmountDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxAmount",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomLineItemTaxAmountAction(**data)


class StagedOrderSetCustomLineItemTaxRateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomLineItemTaxRateAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomLineItemTaxRateAction(**data)


class StagedOrderSetCustomShippingMethodActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomShippingMethodAction`."""

    shipping_method_name = marshmallow.fields.String(
        allow_none=True, data_key="shippingMethodName"
    )
    shipping_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="shippingRate",
    )
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxCategory",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomShippingMethodAction(**data)


class StagedOrderSetCustomTypeActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomTypeAction(**data)


class StagedOrderSetCustomerEmailActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomerEmailAction`."""

    email = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomerEmailAction(**data)


class StagedOrderSetCustomerGroupActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomerGroupAction`."""

    customer_group = helpers.LazyNestedField(
        nested="commercetools._schemas._customer_group.CustomerGroupResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="customerGroup",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomerGroupAction(**data)


class StagedOrderSetCustomerIdActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetCustomerIdAction`."""

    customer_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="customerId"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetCustomerIdAction(**data)


class StagedOrderSetDeliveryAddressActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetDeliveryAddressAction`."""

    delivery_id = marshmallow.fields.String(allow_none=True, data_key="deliveryId")
    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetDeliveryAddressAction(**data)


class StagedOrderSetDeliveryItemsActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetDeliveryItemsAction`."""

    delivery_id = marshmallow.fields.String(allow_none=True, data_key="deliveryId")
    items = helpers.LazyNestedField(
        nested="commercetools._schemas._order.DeliveryItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetDeliveryItemsAction(**data)


class StagedOrderSetLineItemCustomFieldActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLineItemCustomFieldAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLineItemCustomFieldAction(**data)


class StagedOrderSetLineItemCustomTypeActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLineItemCustomTypeAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = FieldContainerField(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLineItemCustomTypeAction(**data)


class StagedOrderSetLineItemDistributionChannelActionSchema(
    StagedOrderUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLineItemDistributionChannelAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    distribution_channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="distributionChannel",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLineItemDistributionChannelAction(**data)


class StagedOrderSetLineItemPriceActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLineItemPriceAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    external_price = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalPrice",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLineItemPriceAction(**data)


class StagedOrderSetLineItemShippingDetailsActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLineItemShippingDetailsAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    shipping_details = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ItemShippingDetailsDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingDetails",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLineItemShippingDetailsAction(**data)


class StagedOrderSetLineItemTaxAmountActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLineItemTaxAmountAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    external_tax_amount = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxAmountDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxAmount",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLineItemTaxAmountAction(**data)


class StagedOrderSetLineItemTaxRateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLineItemTaxRateAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLineItemTaxRateAction(**data)


class StagedOrderSetLineItemTotalPriceActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLineItemTotalPriceAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    external_total_price = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalLineItemTotalPriceSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTotalPrice",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLineItemTotalPriceAction(**data)


class StagedOrderSetLocaleActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetLocaleAction`."""

    locale = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetLocaleAction(**data)


class StagedOrderSetOrderNumberActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetOrderNumberAction`."""

    order_number = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="orderNumber"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetOrderNumberAction(**data)


class StagedOrderSetOrderTotalTaxActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetOrderTotalTaxAction`."""

    external_total_gross = helpers.LazyNestedField(
        nested="commercetools._schemas._common.MoneySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="externalTotalGross",
    )
    external_tax_portions = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.TaxPortionDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
        missing=None,
        data_key="externalTaxPortions",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetOrderTotalTaxAction(**data)


class StagedOrderSetParcelItemsActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetParcelItemsAction`."""

    parcel_id = marshmallow.fields.String(allow_none=True, data_key="parcelId")
    items = helpers.LazyNestedField(
        nested="commercetools._schemas._order.DeliveryItemSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetParcelItemsAction(**data)


class StagedOrderSetParcelMeasurementsActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetParcelMeasurementsAction`."""

    parcel_id = marshmallow.fields.String(allow_none=True, data_key="parcelId")
    measurements = helpers.LazyNestedField(
        nested="commercetools._schemas._order.ParcelMeasurementsSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetParcelMeasurementsAction(**data)


class StagedOrderSetParcelTrackingDataActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetParcelTrackingDataAction`."""

    parcel_id = marshmallow.fields.String(allow_none=True, data_key="parcelId")
    tracking_data = helpers.LazyNestedField(
        nested="commercetools._schemas._order.TrackingDataSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="trackingData",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetParcelTrackingDataAction(**data)


class StagedOrderSetReturnPaymentStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetReturnPaymentStateAction`."""

    return_item_id = marshmallow.fields.String(allow_none=True, data_key="returnItemId")
    payment_state = marshmallow_enum.EnumField(
        types.ReturnPaymentState, by_value=True, data_key="paymentState"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetReturnPaymentStateAction(**data)


class StagedOrderSetReturnShipmentStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetReturnShipmentStateAction`."""

    return_item_id = marshmallow.fields.String(allow_none=True, data_key="returnItemId")
    shipment_state = marshmallow_enum.EnumField(
        types.ReturnShipmentState, by_value=True, data_key="shipmentState"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetReturnShipmentStateAction(**data)


class StagedOrderSetShippingAddressActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetShippingAddressAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetShippingAddressAction(**data)


class StagedOrderSetShippingAddressAndCustomShippingMethodActionSchema(
    StagedOrderUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetShippingAddressAndCustomShippingMethodAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    shipping_method_name = marshmallow.fields.String(
        allow_none=True, data_key="shippingMethodName"
    )
    shipping_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="shippingRate",
    )
    tax_category = helpers.LazyNestedField(
        nested="commercetools._schemas._tax_category.TaxCategoryResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="taxCategory",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetShippingAddressAndCustomShippingMethodAction(**data)


class StagedOrderSetShippingAddressAndShippingMethodActionSchema(
    StagedOrderUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetShippingAddressAndShippingMethodAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    shipping_method = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingMethodResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingMethod",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetShippingAddressAndShippingMethodAction(**data)


class StagedOrderSetShippingMethodActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetShippingMethodAction`."""

    shipping_method = helpers.LazyNestedField(
        nested="commercetools._schemas._shipping_method.ShippingMethodResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingMethod",
    )
    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetShippingMethodAction(**data)


class StagedOrderSetShippingMethodTaxAmountActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetShippingMethodTaxAmountAction`."""

    external_tax_amount = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxAmountDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxAmount",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetShippingMethodTaxAmountAction(**data)


class StagedOrderSetShippingMethodTaxRateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetShippingMethodTaxRateAction`."""

    external_tax_rate = helpers.LazyNestedField(
        nested="commercetools._schemas._cart.ExternalTaxRateDraftSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="externalTaxRate",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetShippingMethodTaxRateAction(**data)


class StagedOrderSetShippingRateInputActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderSetShippingRateInputAction`."""

    shipping_rate_input = helpers.Discriminator(
        discriminator_field=("type", "type"),
        discriminator_schemas={
            "Classification": "commercetools._schemas._cart.ClassificationShippingRateInputDraftSchema",
            "Score": "commercetools._schemas._cart.ScoreShippingRateInputDraftSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="shippingRateInput",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderSetShippingRateInputAction(**data)


class StagedOrderTransitionCustomLineItemStateActionSchema(
    StagedOrderUpdateActionSchema
):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderTransitionCustomLineItemStateAction`."""

    custom_line_item_id = marshmallow.fields.String(
        allow_none=True, data_key="customLineItemId"
    )
    quantity = marshmallow.fields.Integer(allow_none=True)
    from_state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="fromState",
    )
    to_state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="toState",
    )
    actual_transition_date = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="actualTransitionDate"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderTransitionCustomLineItemStateAction(**data)


class StagedOrderTransitionLineItemStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderTransitionLineItemStateAction`."""

    line_item_id = marshmallow.fields.String(allow_none=True, data_key="lineItemId")
    quantity = marshmallow.fields.Integer(allow_none=True)
    from_state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="fromState",
    )
    to_state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="toState",
    )
    actual_transition_date = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="actualTransitionDate"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderTransitionLineItemStateAction(**data)


class StagedOrderTransitionStateActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderTransitionStateAction`."""

    state = helpers.LazyNestedField(
        nested="commercetools._schemas._state.StateResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    force = marshmallow.fields.Bool(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderTransitionStateAction(**data)


class StagedOrderUpdateItemShippingAddressActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderUpdateItemShippingAddressAction`."""

    address = helpers.LazyNestedField(
        nested="commercetools._schemas._common.AddressSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderUpdateItemShippingAddressAction(**data)


class StagedOrderUpdateSyncInfoActionSchema(StagedOrderUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.StagedOrderUpdateSyncInfoAction`."""

    channel = helpers.LazyNestedField(
        nested="commercetools._schemas._channel.ChannelResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    external_id = marshmallow.fields.String(
        allow_none=True, missing=None, data_key="externalId"
    )
    synced_at = marshmallow.fields.DateTime(
        allow_none=True, missing=None, data_key="syncedAt"
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.StagedOrderUpdateSyncInfoAction(**data)


class OrderEditAddStagedActionActionSchema(OrderEditUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditAddStagedActionAction`."""

    staged_action = helpers.Discriminator(
        discriminator_field=("action", "action"),
        discriminator_schemas={
            "addCustomLineItem": "commercetools._schemas._order_edit.StagedOrderAddCustomLineItemActionSchema",
            "addDelivery": "commercetools._schemas._order_edit.StagedOrderAddDeliveryActionSchema",
            "addDiscountCode": "commercetools._schemas._order_edit.StagedOrderAddDiscountCodeActionSchema",
            "addItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderAddItemShippingAddressActionSchema",
            "addLineItem": "commercetools._schemas._order_edit.StagedOrderAddLineItemActionSchema",
            "addParcelToDelivery": "commercetools._schemas._order_edit.StagedOrderAddParcelToDeliveryActionSchema",
            "addPayment": "commercetools._schemas._order_edit.StagedOrderAddPaymentActionSchema",
            "addReturnInfo": "commercetools._schemas._order_edit.StagedOrderAddReturnInfoActionSchema",
            "addShoppingList": "commercetools._schemas._order_edit.StagedOrderAddShoppingListActionSchema",
            "changeCustomLineItemMoney": "commercetools._schemas._order_edit.StagedOrderChangeCustomLineItemMoneyActionSchema",
            "changeCustomLineItemQuantity": "commercetools._schemas._order_edit.StagedOrderChangeCustomLineItemQuantityActionSchema",
            "changeLineItemQuantity": "commercetools._schemas._order_edit.StagedOrderChangeLineItemQuantityActionSchema",
            "changeOrderState": "commercetools._schemas._order_edit.StagedOrderChangeOrderStateActionSchema",
            "changePaymentState": "commercetools._schemas._order_edit.StagedOrderChangePaymentStateActionSchema",
            "changeShipmentState": "commercetools._schemas._order_edit.StagedOrderChangeShipmentStateActionSchema",
            "changeTaxCalculationMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxCalculationModeActionSchema",
            "changeTaxMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxModeActionSchema",
            "changeTaxRoundingMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxRoundingModeActionSchema",
            "importCustomLineItemState": "commercetools._schemas._order_edit.StagedOrderImportCustomLineItemStateActionSchema",
            "importLineItemState": "commercetools._schemas._order_edit.StagedOrderImportLineItemStateActionSchema",
            "removeCustomLineItem": "commercetools._schemas._order_edit.StagedOrderRemoveCustomLineItemActionSchema",
            "removeDelivery": "commercetools._schemas._order_edit.StagedOrderRemoveDeliveryActionSchema",
            "removeDiscountCode": "commercetools._schemas._order_edit.StagedOrderRemoveDiscountCodeActionSchema",
            "removeItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderRemoveItemShippingAddressActionSchema",
            "removeLineItem": "commercetools._schemas._order_edit.StagedOrderRemoveLineItemActionSchema",
            "removeParcelFromDelivery": "commercetools._schemas._order_edit.StagedOrderRemoveParcelFromDeliveryActionSchema",
            "removePayment": "commercetools._schemas._order_edit.StagedOrderRemovePaymentActionSchema",
            "setBillingAddress": "commercetools._schemas._order_edit.StagedOrderSetBillingAddressActionSchema",
            "setCountry": "commercetools._schemas._order_edit.StagedOrderSetCountryActionSchema",
            "setCustomField": "commercetools._schemas._order_edit.StagedOrderSetCustomFieldActionSchema",
            "setCustomLineItemCustomField": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemCustomFieldActionSchema",
            "setCustomLineItemCustomType": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemCustomTypeActionSchema",
            "setCustomLineItemShippingDetails": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemShippingDetailsActionSchema",
            "setCustomLineItemTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemTaxAmountActionSchema",
            "setCustomLineItemTaxRate": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemTaxRateActionSchema",
            "setCustomShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetCustomShippingMethodActionSchema",
            "setCustomType": "commercetools._schemas._order_edit.StagedOrderSetCustomTypeActionSchema",
            "setCustomerEmail": "commercetools._schemas._order_edit.StagedOrderSetCustomerEmailActionSchema",
            "setCustomerGroup": "commercetools._schemas._order_edit.StagedOrderSetCustomerGroupActionSchema",
            "setCustomerId": "commercetools._schemas._order_edit.StagedOrderSetCustomerIdActionSchema",
            "setDeliveryAddress": "commercetools._schemas._order_edit.StagedOrderSetDeliveryAddressActionSchema",
            "setDeliveryItems": "commercetools._schemas._order_edit.StagedOrderSetDeliveryItemsActionSchema",
            "setLineItemCustomField": "commercetools._schemas._order_edit.StagedOrderSetLineItemCustomFieldActionSchema",
            "setLineItemCustomType": "commercetools._schemas._order_edit.StagedOrderSetLineItemCustomTypeActionSchema",
            "setLineItemDistributionChannel": "commercetools._schemas._order_edit.StagedOrderSetLineItemDistributionChannelActionSchema",
            "setLineItemPrice": "commercetools._schemas._order_edit.StagedOrderSetLineItemPriceActionSchema",
            "setLineItemShippingDetails": "commercetools._schemas._order_edit.StagedOrderSetLineItemShippingDetailsActionSchema",
            "setLineItemTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetLineItemTaxAmountActionSchema",
            "setLineItemTaxRate": "commercetools._schemas._order_edit.StagedOrderSetLineItemTaxRateActionSchema",
            "setLineItemTotalPrice": "commercetools._schemas._order_edit.StagedOrderSetLineItemTotalPriceActionSchema",
            "setLocale": "commercetools._schemas._order_edit.StagedOrderSetLocaleActionSchema",
            "setOrderNumber": "commercetools._schemas._order_edit.StagedOrderSetOrderNumberActionSchema",
            "setOrderTotalTax": "commercetools._schemas._order_edit.StagedOrderSetOrderTotalTaxActionSchema",
            "setParcelItems": "commercetools._schemas._order_edit.StagedOrderSetParcelItemsActionSchema",
            "setParcelMeasurements": "commercetools._schemas._order_edit.StagedOrderSetParcelMeasurementsActionSchema",
            "setParcelTrackingData": "commercetools._schemas._order_edit.StagedOrderSetParcelTrackingDataActionSchema",
            "setReturnPaymentState": "commercetools._schemas._order_edit.StagedOrderSetReturnPaymentStateActionSchema",
            "setReturnShipmentState": "commercetools._schemas._order_edit.StagedOrderSetReturnShipmentStateActionSchema",
            "setShippingAddress": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressActionSchema",
            "setShippingAddressAndCustomShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressAndCustomShippingMethodActionSchema",
            "setShippingAddressAndShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressAndShippingMethodActionSchema",
            "setShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodActionSchema",
            "setShippingMethodTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodTaxAmountActionSchema",
            "setShippingMethodTaxRate": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodTaxRateActionSchema",
            "setShippingRateInput": "commercetools._schemas._order_edit.StagedOrderSetShippingRateInputActionSchema",
            "transitionCustomLineItemState": "commercetools._schemas._order_edit.StagedOrderTransitionCustomLineItemStateActionSchema",
            "transitionLineItemState": "commercetools._schemas._order_edit.StagedOrderTransitionLineItemStateActionSchema",
            "transitionState": "commercetools._schemas._order_edit.StagedOrderTransitionStateActionSchema",
            "updateItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderUpdateItemShippingAddressActionSchema",
            "updateSyncInfo": "commercetools._schemas._order_edit.StagedOrderUpdateSyncInfoActionSchema",
        },
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="stagedAction",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.OrderEditAddStagedActionAction(**data)


class OrderEditAppliedSchema(OrderEditResultSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditApplied`."""

    applied_at = marshmallow.fields.DateTime(allow_none=True, data_key="appliedAt")
    excerpt_before_edit = helpers.LazyNestedField(
        nested="commercetools._schemas._order_edit.OrderExcerptSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="excerptBeforeEdit",
    )
    excerpt_after_edit = helpers.LazyNestedField(
        nested="commercetools._schemas._order_edit.OrderExcerptSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        data_key="excerptAfterEdit",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.OrderEditApplied(**data)


class OrderEditNotProcessedSchema(OrderEditResultSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditNotProcessed`."""

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.OrderEditNotProcessed()


class OrderEditPreviewFailureSchema(OrderEditResultSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditPreviewFailure`."""

    errors = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("code", "code"),
            discriminator_schemas={
                "access_denied": "commercetools._schemas._error.AccessDeniedErrorSchema",
                "ConcurrentModification": "commercetools._schemas._error.ConcurrentModificationErrorSchema",
                "DiscountCodeNonApplicable": "commercetools._schemas._error.DiscountCodeNonApplicableErrorSchema",
                "DuplicateAttributeValue": "commercetools._schemas._error.DuplicateAttributeValueErrorSchema",
                "DuplicateAttributeValues": "commercetools._schemas._error.DuplicateAttributeValuesErrorSchema",
                "DuplicateField": "commercetools._schemas._error.DuplicateFieldErrorSchema",
                "DuplicateFieldWithConflictingResource": "commercetools._schemas._error.DuplicateFieldWithConflictingResourceErrorSchema",
                "DuplicatePriceScope": "commercetools._schemas._error.DuplicatePriceScopeErrorSchema",
                "DuplicateVariantValues": "commercetools._schemas._error.DuplicateVariantValuesErrorSchema",
                "EnumValueIsUsed": "commercetools._schemas._error.EnumValueIsUsedErrorSchema",
                "ExtensionBadResponse": "commercetools._schemas._error.ExtensionBadResponseErrorSchema",
                "ExtensionNoResponse": "commercetools._schemas._error.ExtensionNoResponseErrorSchema",
                "ExtensionUpdateActionsFailed": "commercetools._schemas._error.ExtensionUpdateActionsFailedErrorSchema",
                "insufficient_scope": "commercetools._schemas._error.InsufficientScopeErrorSchema",
                "InvalidCredentials": "commercetools._schemas._error.InvalidCredentialsErrorSchema",
                "InvalidCurrentPassword": "commercetools._schemas._error.InvalidCurrentPasswordErrorSchema",
                "InvalidField": "commercetools._schemas._error.InvalidFieldErrorSchema",
                "InvalidInput": "commercetools._schemas._error.InvalidInputErrorSchema",
                "InvalidItemShippingDetails": "commercetools._schemas._error.InvalidItemShippingDetailsErrorSchema",
                "InvalidJsonInput": "commercetools._schemas._error.InvalidJsonInputErrorSchema",
                "InvalidOperation": "commercetools._schemas._error.InvalidOperationErrorSchema",
                "InvalidSubject": "commercetools._schemas._error.InvalidSubjectErrorSchema",
                "invalid_token": "commercetools._schemas._error.InvalidTokenErrorSchema",
                "LanguageUsedInStores": "commercetools._schemas._error.LanguageUsedInStoresErrorSchema",
                "MatchingPriceNotFound": "commercetools._schemas._error.MatchingPriceNotFoundErrorSchema",
                "MissingTaxRateForCountry": "commercetools._schemas._error.MissingTaxRateForCountryErrorSchema",
                "NoMatchingProductDiscountFound": "commercetools._schemas._error.NoMatchingProductDiscountFoundErrorSchema",
                "OutOfStock": "commercetools._schemas._error.OutOfStockErrorSchema",
                "PriceChanged": "commercetools._schemas._error.PriceChangedErrorSchema",
                "ReferenceExists": "commercetools._schemas._error.ReferenceExistsErrorSchema",
                "ReferencedResourceNotFound": "commercetools._schemas._error.ReferencedResourceNotFoundErrorSchema",
                "RequiredField": "commercetools._schemas._error.RequiredFieldErrorSchema",
                "ResourceNotFound": "commercetools._schemas._error.ResourceNotFoundErrorSchema",
                "ShippingMethodDoesNotMatchCart": "commercetools._schemas._error.ShippingMethodDoesNotMatchCartErrorSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        )
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.OrderEditPreviewFailure(**data)


class OrderEditPreviewSuccessSchema(OrderEditResultSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditPreviewSuccess`."""

    preview = helpers.LazyNestedField(
        nested="commercetools._schemas._order_edit.StagedOrderSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
    )
    message_payloads = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("type", "type"),
            discriminator_schemas={
                "CategoryCreated": "commercetools._schemas._message.CategoryCreatedMessagePayloadSchema",
                "CategorySlugChanged": "commercetools._schemas._message.CategorySlugChangedMessagePayloadSchema",
                "CustomLineItemStateTransition": "commercetools._schemas._message.CustomLineItemStateTransitionMessagePayloadSchema",
                "CustomerAddressAdded": "commercetools._schemas._message.CustomerAddressAddedMessagePayloadSchema",
                "CustomerAddressChanged": "commercetools._schemas._message.CustomerAddressChangedMessagePayloadSchema",
                "CustomerAddressRemoved": "commercetools._schemas._message.CustomerAddressRemovedMessagePayloadSchema",
                "CustomerCompanyNameSet": "commercetools._schemas._message.CustomerCompanyNameSetMessagePayloadSchema",
                "CustomerCreated": "commercetools._schemas._message.CustomerCreatedMessagePayloadSchema",
                "CustomerDateOfBirthSet": "commercetools._schemas._message.CustomerDateOfBirthSetMessagePayloadSchema",
                "CustomerEmailChanged": "commercetools._schemas._message.CustomerEmailChangedMessagePayloadSchema",
                "CustomerEmailVerified": "commercetools._schemas._message.CustomerEmailVerifiedMessagePayloadSchema",
                "CustomerGroupSet": "commercetools._schemas._message.CustomerGroupSetMessagePayloadSchema",
                "DeliveryAdded": "commercetools._schemas._message.DeliveryAddedMessagePayloadSchema",
                "DeliveryAddressSet": "commercetools._schemas._message.DeliveryAddressSetMessagePayloadSchema",
                "DeliveryItemsUpdated": "commercetools._schemas._message.DeliveryItemsUpdatedMessagePayloadSchema",
                "DeliveryRemoved": "commercetools._schemas._message.DeliveryRemovedMessagePayloadSchema",
                "InventoryEntryCreated": "commercetools._schemas._message.InventoryEntryCreatedMessagePayloadSchema",
                "InventoryEntryDeleted": "commercetools._schemas._message.InventoryEntryDeletedMessagePayloadSchema",
                "InventoryEntryQuantitySet": "commercetools._schemas._message.InventoryEntryQuantitySetMessagePayloadSchema",
                "LineItemStateTransition": "commercetools._schemas._message.LineItemStateTransitionMessagePayloadSchema",
                "OrderBillingAddressSet": "commercetools._schemas._message.OrderBillingAddressSetMessagePayloadSchema",
                "OrderCreated": "commercetools._schemas._message.OrderCreatedMessagePayloadSchema",
                "OrderCustomLineItemDiscountSet": "commercetools._schemas._message.OrderCustomLineItemDiscountSetMessagePayloadSchema",
                "OrderCustomerEmailSet": "commercetools._schemas._message.OrderCustomerEmailSetMessagePayloadSchema",
                "OrderCustomerGroupSet": "commercetools._schemas._message.OrderCustomerGroupSetMessagePayloadSchema",
                "OrderCustomerSet": "commercetools._schemas._message.OrderCustomerSetMessagePayloadSchema",
                "OrderDeleted": "commercetools._schemas._message.OrderDeletedMessagePayloadSchema",
                "OrderDiscountCodeAdded": "commercetools._schemas._message.OrderDiscountCodeAddedMessagePayloadSchema",
                "OrderDiscountCodeRemoved": "commercetools._schemas._message.OrderDiscountCodeRemovedMessagePayloadSchema",
                "OrderDiscountCodeStateSet": "commercetools._schemas._message.OrderDiscountCodeStateSetMessagePayloadSchema",
                "OrderEditApplied": "commercetools._schemas._message.OrderEditAppliedMessagePayloadSchema",
                "OrderImported": "commercetools._schemas._message.OrderImportedMessagePayloadSchema",
                "OrderLineItemAdded": "commercetools._schemas._message.OrderLineItemAddedMessagePayloadSchema",
                "OrderLineItemDiscountSet": "commercetools._schemas._message.OrderLineItemDiscountSetMessagePayloadSchema",
                "OrderPaymentStateChanged": "commercetools._schemas._message.OrderPaymentStateChangedMessagePayloadSchema",
                "ReturnInfoAdded": "commercetools._schemas._message.OrderReturnInfoAddedMessagePayloadSchema",
                "OrderReturnShipmentStateChanged": "commercetools._schemas._message.OrderReturnShipmentStateChangedMessagePayloadSchema",
                "OrderShipmentStateChanged": "commercetools._schemas._message.OrderShipmentStateChangedMessagePayloadSchema",
                "OrderShippingAddressSet": "commercetools._schemas._message.OrderShippingAddressSetMessagePayloadSchema",
                "OrderShippingInfoSet": "commercetools._schemas._message.OrderShippingInfoSetMessagePayloadSchema",
                "OrderShippingRateInputSet": "commercetools._schemas._message.OrderShippingRateInputSetMessagePayloadSchema",
                "OrderStateChanged": "commercetools._schemas._message.OrderStateChangedMessagePayloadSchema",
                "OrderStateTransition": "commercetools._schemas._message.OrderStateTransitionMessagePayloadSchema",
                "OrderStoreSet": "commercetools._schemas._message.OrderStoreSetMessagePayloadSchema",
                "ParcelAddedToDelivery": "commercetools._schemas._message.ParcelAddedToDeliveryMessagePayloadSchema",
                "ParcelItemsUpdated": "commercetools._schemas._message.ParcelItemsUpdatedMessagePayloadSchema",
                "ParcelMeasurementsUpdated": "commercetools._schemas._message.ParcelMeasurementsUpdatedMessagePayloadSchema",
                "ParcelRemovedFromDelivery": "commercetools._schemas._message.ParcelRemovedFromDeliveryMessagePayloadSchema",
                "ParcelTrackingDataUpdated": "commercetools._schemas._message.ParcelTrackingDataUpdatedMessagePayloadSchema",
                "PaymentCreated": "commercetools._schemas._message.PaymentCreatedMessagePayloadSchema",
                "PaymentInteractionAdded": "commercetools._schemas._message.PaymentInteractionAddedMessagePayloadSchema",
                "PaymentStatusInterfaceCodeSet": "commercetools._schemas._message.PaymentStatusInterfaceCodeSetMessagePayloadSchema",
                "PaymentStatusStateTransition": "commercetools._schemas._message.PaymentStatusStateTransitionMessagePayloadSchema",
                "PaymentTransactionAdded": "commercetools._schemas._message.PaymentTransactionAddedMessagePayloadSchema",
                "PaymentTransactionStateChanged": "commercetools._schemas._message.PaymentTransactionStateChangedMessagePayloadSchema",
                "ProductAddedToCategory": "commercetools._schemas._message.ProductAddedToCategoryMessagePayloadSchema",
                "ProductCreated": "commercetools._schemas._message.ProductCreatedMessagePayloadSchema",
                "ProductDeleted": "commercetools._schemas._message.ProductDeletedMessagePayloadSchema",
                "ProductImageAdded": "commercetools._schemas._message.ProductImageAddedMessagePayloadSchema",
                "ProductPriceDiscountsSet": "commercetools._schemas._message.ProductPriceDiscountsSetMessagePayloadSchema",
                "ProductPriceExternalDiscountSet": "commercetools._schemas._message.ProductPriceExternalDiscountSetMessagePayloadSchema",
                "ProductPublished": "commercetools._schemas._message.ProductPublishedMessagePayloadSchema",
                "ProductRemovedFromCategory": "commercetools._schemas._message.ProductRemovedFromCategoryMessagePayloadSchema",
                "ProductRevertedStagedChanges": "commercetools._schemas._message.ProductRevertedStagedChangesMessagePayloadSchema",
                "ProductSlugChanged": "commercetools._schemas._message.ProductSlugChangedMessagePayloadSchema",
                "ProductStateTransition": "commercetools._schemas._message.ProductStateTransitionMessagePayloadSchema",
                "ProductUnpublished": "commercetools._schemas._message.ProductUnpublishedMessagePayloadSchema",
                "ProductVariantDeleted": "commercetools._schemas._message.ProductVariantDeletedMessagePayloadSchema",
                "ReviewCreated": "commercetools._schemas._message.ReviewCreatedMessagePayloadSchema",
                "ReviewRatingSet": "commercetools._schemas._message.ReviewRatingSetMessagePayloadSchema",
                "ReviewStateTransition": "commercetools._schemas._message.ReviewStateTransitionMessagePayloadSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        data_key="messagePayloads",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type"]
        return types.OrderEditPreviewSuccess(**data)


class OrderEditSetCommentActionSchema(OrderEditUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditSetCommentAction`."""

    comment = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.OrderEditSetCommentAction(**data)


class OrderEditSetCustomFieldActionSchema(OrderEditUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditSetCustomFieldAction`."""

    name = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.OrderEditSetCustomFieldAction(**data)


class OrderEditSetCustomTypeActionSchema(OrderEditUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditSetCustomTypeAction`."""

    type = helpers.LazyNestedField(
        nested="commercetools._schemas._type.TypeResourceIdentifierSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )
    fields = marshmallow.fields.Dict(allow_none=True, missing=None)  # type: ignore

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.OrderEditSetCustomTypeAction(**data)


class OrderEditSetKeyActionSchema(OrderEditUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditSetKeyAction`."""

    key = marshmallow.fields.String(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.OrderEditSetKeyAction(**data)


class OrderEditSetStagedActionsActionSchema(OrderEditUpdateActionSchema):
    """Marshmallow schema for :class:`commercetools.types.OrderEditSetStagedActionsAction`."""

    staged_actions = marshmallow.fields.List(
        helpers.Discriminator(
            discriminator_field=("action", "action"),
            discriminator_schemas={
                "addCustomLineItem": "commercetools._schemas._order_edit.StagedOrderAddCustomLineItemActionSchema",
                "addDelivery": "commercetools._schemas._order_edit.StagedOrderAddDeliveryActionSchema",
                "addDiscountCode": "commercetools._schemas._order_edit.StagedOrderAddDiscountCodeActionSchema",
                "addItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderAddItemShippingAddressActionSchema",
                "addLineItem": "commercetools._schemas._order_edit.StagedOrderAddLineItemActionSchema",
                "addParcelToDelivery": "commercetools._schemas._order_edit.StagedOrderAddParcelToDeliveryActionSchema",
                "addPayment": "commercetools._schemas._order_edit.StagedOrderAddPaymentActionSchema",
                "addReturnInfo": "commercetools._schemas._order_edit.StagedOrderAddReturnInfoActionSchema",
                "addShoppingList": "commercetools._schemas._order_edit.StagedOrderAddShoppingListActionSchema",
                "changeCustomLineItemMoney": "commercetools._schemas._order_edit.StagedOrderChangeCustomLineItemMoneyActionSchema",
                "changeCustomLineItemQuantity": "commercetools._schemas._order_edit.StagedOrderChangeCustomLineItemQuantityActionSchema",
                "changeLineItemQuantity": "commercetools._schemas._order_edit.StagedOrderChangeLineItemQuantityActionSchema",
                "changeOrderState": "commercetools._schemas._order_edit.StagedOrderChangeOrderStateActionSchema",
                "changePaymentState": "commercetools._schemas._order_edit.StagedOrderChangePaymentStateActionSchema",
                "changeShipmentState": "commercetools._schemas._order_edit.StagedOrderChangeShipmentStateActionSchema",
                "changeTaxCalculationMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxCalculationModeActionSchema",
                "changeTaxMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxModeActionSchema",
                "changeTaxRoundingMode": "commercetools._schemas._order_edit.StagedOrderChangeTaxRoundingModeActionSchema",
                "importCustomLineItemState": "commercetools._schemas._order_edit.StagedOrderImportCustomLineItemStateActionSchema",
                "importLineItemState": "commercetools._schemas._order_edit.StagedOrderImportLineItemStateActionSchema",
                "removeCustomLineItem": "commercetools._schemas._order_edit.StagedOrderRemoveCustomLineItemActionSchema",
                "removeDelivery": "commercetools._schemas._order_edit.StagedOrderRemoveDeliveryActionSchema",
                "removeDiscountCode": "commercetools._schemas._order_edit.StagedOrderRemoveDiscountCodeActionSchema",
                "removeItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderRemoveItemShippingAddressActionSchema",
                "removeLineItem": "commercetools._schemas._order_edit.StagedOrderRemoveLineItemActionSchema",
                "removeParcelFromDelivery": "commercetools._schemas._order_edit.StagedOrderRemoveParcelFromDeliveryActionSchema",
                "removePayment": "commercetools._schemas._order_edit.StagedOrderRemovePaymentActionSchema",
                "setBillingAddress": "commercetools._schemas._order_edit.StagedOrderSetBillingAddressActionSchema",
                "setCountry": "commercetools._schemas._order_edit.StagedOrderSetCountryActionSchema",
                "setCustomField": "commercetools._schemas._order_edit.StagedOrderSetCustomFieldActionSchema",
                "setCustomLineItemCustomField": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemCustomFieldActionSchema",
                "setCustomLineItemCustomType": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemCustomTypeActionSchema",
                "setCustomLineItemShippingDetails": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemShippingDetailsActionSchema",
                "setCustomLineItemTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemTaxAmountActionSchema",
                "setCustomLineItemTaxRate": "commercetools._schemas._order_edit.StagedOrderSetCustomLineItemTaxRateActionSchema",
                "setCustomShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetCustomShippingMethodActionSchema",
                "setCustomType": "commercetools._schemas._order_edit.StagedOrderSetCustomTypeActionSchema",
                "setCustomerEmail": "commercetools._schemas._order_edit.StagedOrderSetCustomerEmailActionSchema",
                "setCustomerGroup": "commercetools._schemas._order_edit.StagedOrderSetCustomerGroupActionSchema",
                "setCustomerId": "commercetools._schemas._order_edit.StagedOrderSetCustomerIdActionSchema",
                "setDeliveryAddress": "commercetools._schemas._order_edit.StagedOrderSetDeliveryAddressActionSchema",
                "setDeliveryItems": "commercetools._schemas._order_edit.StagedOrderSetDeliveryItemsActionSchema",
                "setLineItemCustomField": "commercetools._schemas._order_edit.StagedOrderSetLineItemCustomFieldActionSchema",
                "setLineItemCustomType": "commercetools._schemas._order_edit.StagedOrderSetLineItemCustomTypeActionSchema",
                "setLineItemDistributionChannel": "commercetools._schemas._order_edit.StagedOrderSetLineItemDistributionChannelActionSchema",
                "setLineItemPrice": "commercetools._schemas._order_edit.StagedOrderSetLineItemPriceActionSchema",
                "setLineItemShippingDetails": "commercetools._schemas._order_edit.StagedOrderSetLineItemShippingDetailsActionSchema",
                "setLineItemTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetLineItemTaxAmountActionSchema",
                "setLineItemTaxRate": "commercetools._schemas._order_edit.StagedOrderSetLineItemTaxRateActionSchema",
                "setLineItemTotalPrice": "commercetools._schemas._order_edit.StagedOrderSetLineItemTotalPriceActionSchema",
                "setLocale": "commercetools._schemas._order_edit.StagedOrderSetLocaleActionSchema",
                "setOrderNumber": "commercetools._schemas._order_edit.StagedOrderSetOrderNumberActionSchema",
                "setOrderTotalTax": "commercetools._schemas._order_edit.StagedOrderSetOrderTotalTaxActionSchema",
                "setParcelItems": "commercetools._schemas._order_edit.StagedOrderSetParcelItemsActionSchema",
                "setParcelMeasurements": "commercetools._schemas._order_edit.StagedOrderSetParcelMeasurementsActionSchema",
                "setParcelTrackingData": "commercetools._schemas._order_edit.StagedOrderSetParcelTrackingDataActionSchema",
                "setReturnPaymentState": "commercetools._schemas._order_edit.StagedOrderSetReturnPaymentStateActionSchema",
                "setReturnShipmentState": "commercetools._schemas._order_edit.StagedOrderSetReturnShipmentStateActionSchema",
                "setShippingAddress": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressActionSchema",
                "setShippingAddressAndCustomShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressAndCustomShippingMethodActionSchema",
                "setShippingAddressAndShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingAddressAndShippingMethodActionSchema",
                "setShippingMethod": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodActionSchema",
                "setShippingMethodTaxAmount": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodTaxAmountActionSchema",
                "setShippingMethodTaxRate": "commercetools._schemas._order_edit.StagedOrderSetShippingMethodTaxRateActionSchema",
                "setShippingRateInput": "commercetools._schemas._order_edit.StagedOrderSetShippingRateInputActionSchema",
                "transitionCustomLineItemState": "commercetools._schemas._order_edit.StagedOrderTransitionCustomLineItemStateActionSchema",
                "transitionLineItemState": "commercetools._schemas._order_edit.StagedOrderTransitionLineItemStateActionSchema",
                "transitionState": "commercetools._schemas._order_edit.StagedOrderTransitionStateActionSchema",
                "updateItemShippingAddress": "commercetools._schemas._order_edit.StagedOrderUpdateItemShippingAddressActionSchema",
                "updateSyncInfo": "commercetools._schemas._order_edit.StagedOrderUpdateSyncInfoActionSchema",
            },
            unknown=marshmallow.EXCLUDE,
            allow_none=True,
        ),
        data_key="stagedActions",
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["action"]
        return types.OrderEditSetStagedActionsAction(**data)



# DO NOT EDIT! This file is automatically generated
import marshmallow

from commercetools import helpers, types
from commercetools._schemas._common import BaseResourceSchema, ReferenceSchema

__all__ = [
    "CustomObjectDraftSchema",
    "CustomObjectPagedQueryResponseSchema",
    "CustomObjectReferenceSchema",
    "CustomObjectSchema",
]


class CustomObjectDraftSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CustomObjectDraft`."""

    container = marshmallow.fields.String(allow_none=True)
    key = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True, missing=None)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomObjectDraft(**data)


class CustomObjectPagedQueryResponseSchema(marshmallow.Schema):
    """Marshmallow schema for :class:`commercetools.types.CustomObjectPagedQueryResponse`."""

    limit = marshmallow.fields.Integer(allow_none=True)
    count = marshmallow.fields.Integer(allow_none=True)
    total = marshmallow.fields.Integer(allow_none=True, missing=None)
    offset = marshmallow.fields.Integer(allow_none=True)
    results = helpers.LazyNestedField(
        nested="commercetools._schemas._custom_object.CustomObjectSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        many=True,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomObjectPagedQueryResponse(**data)


class CustomObjectReferenceSchema(ReferenceSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomObjectReference`."""

    obj = helpers.LazyNestedField(
        nested="commercetools._schemas._custom_object.CustomObjectSchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
    )

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        del data["type_id"]
        return types.CustomObjectReference(**data)


class CustomObjectSchema(BaseResourceSchema):
    """Marshmallow schema for :class:`commercetools.types.CustomObject`."""

    id = marshmallow.fields.String(allow_none=True)
    version = marshmallow.fields.Integer(allow_none=True)
    created_at = marshmallow.fields.DateTime(allow_none=True, data_key="createdAt")
    last_modified_at = marshmallow.fields.DateTime(
        allow_none=True, data_key="lastModifiedAt"
    )
    last_modified_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.LastModifiedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="lastModifiedBy",
    )
    created_by = helpers.LazyNestedField(
        nested="commercetools._schemas._common.CreatedBySchema",
        unknown=marshmallow.EXCLUDE,
        allow_none=True,
        missing=None,
        data_key="createdBy",
    )
    container = marshmallow.fields.String(allow_none=True)
    key = marshmallow.fields.String(allow_none=True)
    value = marshmallow.fields.Raw(allow_none=True)

    class Meta:
        unknown = marshmallow.EXCLUDE

    @marshmallow.post_load
    def post_load(self, data, **kwargs):
        return types.CustomObject(**data)



import pytest
from requests.exceptions import HTTPError

from commercetools import types


def test_shipping_method_get_by_id(client):
    shipping_method = client.shipping_methods.create(
        types.ShippingMethodDraft(
            key="test-shipping-method",
            name="test shipping method",
            tax_category=types.TaxCategoryResourceIdentifier(id="dummy"),
            zone_rates=[],
            is_default=False,
        )
    )

    assert shipping_method.id
    assert shipping_method.key == "test-shipping-method"
    assert shipping_method.name == "test shipping method"

    shipping_method = client.shipping_methods.get_by_id(shipping_method.id)
    assert shipping_method.id
    assert shipping_method.key == "test-shipping-method"

    with pytest.raises(HTTPError):
        client.shipping_methods.get_by_id("invalid")


def test_shipping_method_query(client):
    client.shipping_methods.create(
        types.ShippingMethodDraft(
            key="test-shipping_method1",
            name="test shipping method1",
            tax_category=types.TaxCategoryResourceIdentifier(id="dummy"),
            zone_rates=[],
            is_default=False,
        )
    )
    client.shipping_methods.create(
        types.ShippingMethodDraft(
            key="test-shipping_method2",
            name="test shipping method2",
            tax_category=types.TaxCategoryResourceIdentifier(id="dummy"),
            zone_rates=[],
            is_default=False,
        )
    )

    # single sort query
    result = client.shipping_methods.query(sort="id asc")
    assert len(result.results) == 2
    assert result.total == 2

    # multiple sort queries
    result = client.shipping_methods.query(sort=["id asc", "name asc"])
    assert len(result.results) == 2
    assert result.total == 2


def test_shipping_method_update(client):
    """Test the return value of the update methods.

    It doesn't test the actual update itself.
    TODO: See if this is worth testing since we're using a mocking backend
    """
    shipping_method = client.shipping_methods.create(
        types.ShippingMethodDraft(
            key="test-shipping-method",
            name="test shipping method",
            tax_category=types.TaxCategoryResourceIdentifier(id="dummy"),
            zone_rates=[],
            is_default=False,
        )
    )
    assert shipping_method.key == "test-shipping-method"

    shipping_method = client.shipping_methods.update_by_id(
        id=shipping_method.id,
        version=shipping_method.version,
        actions=[types.ShippingMethodChangeNameAction(name="shipping-method-2")],
    )

    assert shipping_method.key == "test-shipping-method"



# DO NOT EDIT! This file is automatically generated
import datetime
import typing

from commercetools.types._abstract import _BaseType

__all__ = [
    "GraphQLError",
    "GraphQLErrorLocation",
    "GraphQLRequest",
    "GraphQLResponse",
    "GraphQLVariablesMap",
]


class GraphQLError(_BaseType):
    #: :class:`str`
    message: str
    #: :class:`list`
    locations: list
    #: :class:`list`
    path: list

    def __init__(self, *, message: str, locations: list, path: list) -> None:
        self.message = message
        self.locations = locations
        self.path = path
        super().__init__()

    def __repr__(self) -> str:
        return "GraphQLError(message=%r, locations=%r, path=%r)" % (
            self.message,
            self.locations,
            self.path,
        )


class GraphQLErrorLocation(_BaseType):
    #: :class:`int`
    line: int
    #: :class:`int`
    column: int

    def __init__(self, *, line: int, column: int) -> None:
        self.line = line
        self.column = column
        super().__init__()

    def __repr__(self) -> str:
        return "GraphQLErrorLocation(line=%r, column=%r)" % (self.line, self.column)


class GraphQLRequest(_BaseType):
    #: :class:`str`
    query: str
    #: Optional :class:`str` `(Named` ``operationName`` `in Commercetools)`
    operation_name: typing.Optional[str]
    #: Optional :class:`commercetools.types.GraphQLVariablesMap`
    variables: typing.Optional["GraphQLVariablesMap"]

    def __init__(
        self,
        *,
        query: str,
        operation_name: typing.Optional[str] = None,
        variables: typing.Optional["GraphQLVariablesMap"] = None,
    ) -> None:
        self.query = query
        self.operation_name = operation_name
        self.variables = variables
        super().__init__()

    def __repr__(self) -> str:
        return "GraphQLRequest(query=%r, operation_name=%r, variables=%r)" % (
            self.query,
            self.operation_name,
            self.variables,
        )


class GraphQLResponse(_BaseType):
    #: Optional :class:`typing.Any`
    data: typing.Optional[typing.Any]
    #: Optional :class:`list`
    errors: typing.Optional[list]

    def __init__(
        self,
        *,
        data: typing.Optional[typing.Any] = None,
        errors: typing.Optional[list] = None,
    ) -> None:
        self.data = data
        self.errors = errors
        super().__init__()

    def __repr__(self) -> str:
        return "GraphQLResponse(data=%r, errors=%r)" % (self.data, self.errors)


class GraphQLVariablesMap(typing.Dict[str, typing.Any]):
    def __repr__(self) -> str:
        return "GraphQLVariablesMap(%s)" % (
            ", ".join(f"{k}={v!r}" for k, v in self.items())
        )



import copy
import datetime
import typing
import uuid

from commercetools import types
from commercetools._schemas._shipping_method import (
    ShippingMethodDraftSchema,
    ShippingMethodPagedQueryResponseSchema,
    ShippingMethodSchema,
    ShippingMethodUpdateSchema,
    ShippingRateSchema,
    ZoneRateSchema,
)
from commercetools._schemas._tax_category import TaxCategoryResourceIdentifierSchema
from commercetools.testing import utils
from commercetools.testing.abstract import BaseModel, ServiceBackend
from commercetools.testing.utils import InternalUpdateError, update_attribute


class ShippingMethodsModel(BaseModel):
    _primary_type_name = "shipping-method"
    _resource_schema = ShippingMethodSchema
    _schema_update = ShippingMethodUpdateSchema
    _unique_values = ["key"]

    def _create_from_draft(
        self, draft: types.ShippingMethodDraft, id: typing.Optional[str] = None
    ) -> types.ShippingMethod:
        object_id = str(uuid.UUID(id) if id is not None else uuid.uuid4())

        zone_rates: typing.List[types.ZoneRate] = []
        for rate_draft in draft.zone_rates or []:
            shipping_rates: typing.List[types.ShippingRate] = []
            for shipping_rate_draft in rate_draft.shipping_rates or []:
                shipping_rate = types.ShippingRate(
                    price=utils._money_to_typed(shipping_rate_draft.price),
                    free_above=utils._money_to_typed(shipping_rate_draft.free_above),
                    tiers=copy.deepcopy(shipping_rate_draft.tiers),
                )
                shipping_rates.append(shipping_rate)

            zone_rate = types.ZoneRate(
                zone=rate_draft.zone, shipping_rates=shipping_rates
            )
            zone_rates.append(zone_rate)

        return types.ShippingMethod(
            id=str(object_id),
            key=draft.key,
            version=1,
            created_at=datetime.datetime.now(datetime.timezone.utc),
            last_modified_at=datetime.datetime.now(datetime.timezone.utc),
            name=draft.name,
            description=draft.description,
            tax_category=draft.tax_category,
            zone_rates=zone_rates,
            is_default=draft.is_default,
            predicate=draft.predicate,
        )


def create_shipping_rate_from_draft(
    draft: types.ShippingRateDraft,
) -> types.ShippingRate:
    free_above = None
    if draft.free_above:
        free_above = types.CentPrecisionMoney(
            cent_amount=draft.free_above.cent_amount,
            currency_code=draft.free_above.currency_code,
            fraction_digits=2,
        )
    shipping_rate = types.ShippingRate(
        price=types.CentPrecisionMoney(
            cent_amount=draft.price.cent_amount,
            currency_code=draft.price.currency_code,
            fraction_digits=2,
        ),
        free_above=free_above,
        tiers=draft.tiers if draft.tiers else [],
    )
    return shipping_rate


def change_is_default(
    backend: "ShippingMethodsBackend",
    obj: dict,
    action: types.ShippingMethodChangeIsDefaultAction,
):
    if action.is_default:
        for shipping_method in backend.model._storage._stores[
            "shipping-method"
        ].values():
            shipping_method["isDefault"] = False
            backend.model.save(shipping_method)
    new = copy.deepcopy(obj)
    new["isDefault"] = action.is_default
    return new


def change_tax_category(
    backend: "ShippingMethodsBackend",
    obj: dict,
    action: types.ShippingMethodChangeTaxCategoryAction,
):
    for tax_category in backend.model._storage._stores["tax-category"].values():
        if action.tax_category.id and tax_category["id"] == action.tax_category.id:
            break
        if action.tax_category.key and tax_category["key"] == action.tax_category.key:
            break
    else:
        raise InternalUpdateError("Tax Category does not exist")
    new = copy.deepcopy(obj)
    new["taxCategory"] = TaxCategoryResourceIdentifierSchema().dump(action.tax_category)
    return new


def add_shipping_zone(
    backend: "ShippingMethodsBackend",
    obj: dict,
    action: types.ShippingMethodAddZoneAction,
):
    new = copy.deepcopy(obj)
    if not new.get("zoneRates"):
        new["zoneRates"] = []
    for zone_rate in new["zoneRates"]:
        if zone_rate["zone"]["id"] == action.zone.id:
            raise InternalUpdateError("Zone already exists")

    zone_rate = ZoneRateSchema().dump(
        types.ZoneRate(zone=action.zone, shipping_rates=[])
    )
    new["zoneRates"].append(zone_rate)
    return new


def remove_shipping_zone(
    backend: "ShippingMethodsBackend",
    obj: dict,
    action: types.ShippingMethodRemoveZoneAction,
):
    new = copy.deepcopy(obj)
    for zone_rate in new["zoneRates"]:
        if zone_rate["zone"]["id"] == action.zone.id:
            new["zoneRates"].remove(zone_rate)
            break
    else:
        raise InternalUpdateError("Zone rate not found")

    return new


def add_shipping_rate(
    backend: "ShippingMethodsBackend",
    obj: dict,
    action: types.ShippingMethodAddShippingRateAction,
):
    new = copy.deepcopy(obj)
    draft = action.shipping_rate
    shipping_rate = create_shipping_rate_from_draft(draft)
    target_zone_rate = None
    for zone_rate in new["zoneRates"]:
        if zone_rate["zone"]["id"] == action.zone.id:
            target_zone_rate = zone_rate
            break
    if not target_zone_rate:
        new["zoneRates"].append(
            ZoneRateSchema().dump(
                types.ZoneRate(zone=action.zone, shipping_rates=[shipping_rate])
            )
        )
    else:
        if not target_zone_rate.get("shippingRates"):
            target_zone_rate["shippingRates"] = []
        target_zone_rate["shippingRates"].append(
            ShippingRateSchema().dump(shipping_rate)
        )

    return new


def remove_shipping_rate(
    backend: "ShippingMethodsBackend",
    obj: dict,
    action: types.ShippingMethodRemoveShippingRateAction,
):
    target_zone_rate = None
    for zone_rate in obj["zoneRates"]:
        if zone_rate["zone"]["id"] == action.zone.id:
            target_zone_rate = zone_rate
            break

    if not target_zone_rate:
        raise InternalUpdateError("Zone does not exist")

    rate_to_delete = create_shipping_rate_from_draft(action.shipping_rate)
    for shipping_rate in target_zone_rate["shippingRates"]:
        existing_rate = ShippingRateSchema().load(shipping_rate)
        if existing_rate == rate_to_delete:
            target_zone_rate["shippingRates"].remove(shipping_rate)
            return copy.deepcopy(obj)

    raise InternalUpdateError("Shipping rate does not exist")


class ShippingMethodsBackend(ServiceBackend):
    service_path = "shipping-methods"
    model_class = ShippingMethodsModel
    _schema_draft = ShippingMethodDraftSchema
    _schema_update = ShippingMethodUpdateSchema
    _schema_query_response = ShippingMethodPagedQueryResponseSchema

    def urls(self):
        return [
            ("^$", "GET", self.query),
            ("^$", "POST", self.create),
            ("^(?P<id>[^/]+)$", "GET", self.get_by_id),
            ("^key=(?P<key>[^/]+)$", "GET", self.get_by_key),
            ("^(?P<id>[^/]+)$", "POST", self.update_by_id),
            ("^key=(?P<key>[^/]+)$", "POST", self.update_by_key),
            ("^(?P<id>[^/]+)$", "DELETE", self.delete_by_id),
            ("^key=(?P<key>[^/]+)$", "DELETE", self.delete_by_key),
        ]

    _actions = {
        "addShippingRate": add_shipping_rate,
        "addZone": add_shipping_zone,
        "changeIsDefault": change_is_default,
        "changeName": update_attribute("name", "name"),
        "changeTaxCategory": change_tax_category,
        "removeShippingRate": remove_shipping_rate,
        "removeZone": remove_shipping_zone,
        "setDescription": update_attribute("description", "description"),
        "setKey": update_attribute("key", "key"),
        "setPredicate": update_attribute("predicate", "predicate"),
    }
